# -*- coding: utf-8 -*-
"""This file contains the Windows NT Language identifiers."""

LANGUAGE_IDENTIFIERS = {
    u'af': [0x0036, u'Afrikaans'],
    u'af-ZA': [0x0436, u'Afrikaans, South Africa'],
    u'am': [0x005e, u'Amharic'],
    u'am-ET': [0x045e, u'Amharic, Ethiopia'],
    u'ar': [0x0001, u'Arabic'],
    u'ar-AE': [0x3801, u'Arabic, United Arab Emirates'],
    u'ar-BH': [0x3c01, u'Arabic, Bahrain'],
    u'ar-DZ': [0x1401, u'Arabic, Algeria'],
    u'ar-EG': [0x0c01, u'Arabic, Egypt'],
    u'ar-IQ': [0x0801, u'Arabic, Iraq'],
    u'ar-JO': [0x2c01, u'Arabic, Jordan'],
    u'ar-KW': [0x3401, u'Arabic, Kuwait'],
    u'ar-LB': [0x3001, u'Arabic, Lebanon'],
    u'ar-LY': [0x1001, u'Arabic, Libyan Arab Jamahiriya'],
    u'ar-MA': [0x1801, u'Arabic, Morocco'],
    u'arn': [0x007a, u'Mapudungun'],
    u'arn-CL': [0x047a, u'Mapudungun, Chile'],
    u'ar-OM': [0x2001, u'Arabic, Oman'],
    u'ar-QA': [0x4001, u'Arabic, Qatar'],
    u'ar-SA': [0x0401, u'Arabic, Saudi Arabia'],
    u'ar-SY': [0x2801, u'Arabic, Syrian Arab Republic'],
    u'ar-TN': [0x1c01, u'Arabic, Tunisia'],
    u'ar-YE': [0x2401, u'Arabic, Yemen'],
    u'as': [0x004d, u'Assamese'],
    u'as-IN': [0x044d, u'Assamese, India'],
    u'az': [0x002c, u'Azerbaijani'],
    u'az-Cyrl': [0x742c, u'Azerbaijani, Cyrillic'],
    u'az-Cyrl-AZ': [0x082c, u'Azerbaijani, Cyrillic, Azerbaijan'],
    u'az-Latn': [0x782c, u'Azerbaijani, Latin'],
    u'az-Latn-AZ': [0x042c, u'Azerbaijani, Latin, Azerbaijan'],
    u'ba': [0x006d, u'Bashkir'],
    u'ba-RU': [0x046d, u'Bashkir, Russian Federation'],
    u'be': [0x0023, u'Belarusian'],
    u'be-BY': [0x0423, u'Belarusian, Belarus'],
    u'bg': [0x0002, u'Bulgarian'],
    u'bg-BG': [0x0402, u'Bulgarian, Bulgaria'],
    u'bin-NG': [0x0466, u'Bini, Nigeria'],
    u'bn': [0x0045, u'Bengali'],
    u'bn-BD': [0x0845, u'Bengali, Bangladesh'],
    u'bn-IN': [0x0445, u'Bengali, India'],
    u'bo': [0x0051, u'Tibetan'],
    u'bo-BT': [0x0851, u'Tibetan, Bhutan'],
    u'bo-CN': [0x0451, u'Tibetan, China'],
    u'br': [0x007e, u'Breton'],
    u'br-FR': [0x047e, u'Breton, France'],
    u'bs': [0x781a, u'Bosnian'],
    u'bs-Cyrl': [0x641a, u'Bosnian, Cyrillic'],
    u'bs-Cyrl-BA': [0x201a, u'Bosnian, Cyrillic, Bosnia and Herzegovina'],
    u'bs-Latn': [0x681a, u'Bosnian, Latin'],
    u'bs-Latn-BA': [0x141a, u'Bosnian, Latin, Bosnia and Herzegovina'],
    u'ca': [0x0003, u'Catalan'],
    u'ca-ES': [0x0403, u'Catalan, Spain'],
    u'chr-US': [0x045c, u'Cherokee, United States'],
    u'co': [0x0083, u'Corsican'],
    u'co-FR': [0x0483, u'Corsican, France'],
    u'cs': [0x0005, u'Czech'],
    u'cs-CZ': [0x0405, u'Czech, Czech Republic'],
    u'cy': [0x0052, u'Welsh'],
    u'cy-GB': [0x0452, u'Welsh, United Kingdom'],
    u'da': [0x0006, u'Danish'],
    u'da-DK': [0x0406, u'Danish, Denmark'],
    u'de': [0x0007, u'German'],
    u'de-AT': [0x0c07, u'German, Austria'],
    u'de-CH': [0x0807, u'German, Switzerland'],
    u'de-DE': [0x0407, u'German, Germany'],
    u'de-LI': [0x1407, u'German, Liechtenstein'],
    u'de-LU': [0x1007, u'German, Luxembourg'],
    u'dsb': [0x7c2e, u'Lower Sorbian'],
    u'dsb-DE': [0x082e, u'Lower Sorbian, Germany'],
    u'dv': [0x0065, u'Dhivehi'],
    u'dv-MV': [0x0465, u'Dhivehi, Maldives'],
    u'el': [0x0008, u'Modern Greek (1453 and later)'],
    u'el-GR': [0x0408, u'Modern Greek (1453-), Greece'],
    u'en': [0x0009, u'English'],
    u'en-AU': [0x0c09, u'English, Australia'],
    u'en-BZ': [0x2809, u'English, Belize'],
    u'en-CA': [0x1009, u'English, Canada'],
    u'en-CB': [0x2409, u'English'],
    u'en-GB': [0x0809, u'English, United Kingdom'],
    u'en-HK': [0x3c09, u'English, Hong Kong'],
    u'en-ID': [0x3809, u'English, Indonesia'],
    u'en-IE': [0x1809, u'English, Ireland'],
    u'en-IN': [0x4009, u'English, India'],
    u'en-JM': [0x2009, u'English, Jamaica'],
    u'en-MY': [0x4409, u'English, Malaysia'],
    u'en-NZ': [0x1409, u'English, New Zealand'],
    u'en-PH': [0x3409, u'English, Philippines'],
    u'en-SG': [0x4809, u'English, Singapore'],
    u'en-TT': [0x2c09, u'English, Trinidad and Tobago'],
    u'en-US': [0x0409, u'English, United States'],
    u'en-ZA': [0x1c09, u'English, South Africa'],
    u'en-ZW': [0x3009, u'English, Zimbabwe'],
    u'es': [0x000a, u'Spanish'],
    u'es-AR': [0x2c0a, u'Spanish, Argentina'],
    u'es-BO': [0x400a, u'Spanish, Bolivia'],
    u'es-CL': [0x340a, u'Spanish, Chile'],
    u'es-CO': [0x240a, u'Spanish, Colombia'],
    u'es-CR': [0x140a, u'Spanish, Costa Rica'],
    u'es-DO': [0x1c0a, u'Spanish, Dominican Republic'],
    u'es-EC': [0x300a, u'Spanish, Ecuador'],
    u'es-ES': [0x0c0a, u'Spanish, Spain'],
    u'es-ES_tradnl': [0x040a, u'Spanish'],
    u'es-GT': [0x100a, u'Spanish, Guatemala'],
    u'es-HN': [0x480a, u'Spanish, Honduras'],
    u'es-MX': [0x080a, u'Spanish, Mexico'],
    u'es-NI': [0x4c0a, u'Spanish, Nicaragua'],
    u'es-PA': [0x180a, u'Spanish, Panama'],
    u'es-PE': [0x280a, u'Spanish, Peru'],
    u'es-PR': [0x500a, u'Spanish, Puerto Rico'],
    u'es-PY': [0x3c0a, u'Spanish, Paraguay'],
    u'es-SV': [0x440a, u'Spanish, El Salvador'],
    u'es-US': [0x540a, u'Spanish, United States'],
    u'es-UY': [0x380a, u'Spanish, Uruguay'],
    u'es-VE': [0x200a, u'Spanish, Venezuela'],
    u'et': [0x0025, u'Estonian'],
    u'et-EE': [0x0425, u'Estonian, Estonia'],
    u'eu': [0x002d, u'Basque'],
    u'eu-ES': [0x042d, u'Basque, Spain'],
    u'fa': [0x0029, u'Persian'],
    u'fa-IR': [0x0429, u'Persian, Islamic Republic of Iran'],
    u'fi': [0x000b, u'Finnish'],
    u'fi-FI': [0x040b, u'Finnish, Finland'],
    u'fil': [0x0064, u'Filipino'],
    u'fil-PH': [0x0464, u'Filipino, Philippines'],
    u'fo': [0x0038, u'Faroese'],
    u'fo-FO': [0x0438, u'Faroese, Faroe Islands'],
    u'fr': [0x000c, u'French'],
    u'fr-BE': [0x080c, u'French, Belgium'],
    u'fr-CA': [0x0c0c, u'French, Canada'],
    u'fr-CG': [0x240c, u'French, Congo'],
    u'fr-CH': [0x100c, u'French, Switzerland'],
    u'fr-CI': [0x300c, u'French, Côte d\'Ivoire'],
    u'fr-CM': [0x2c0c, u'French, Cameroon'],
    u'fr-FR': [0x040c, u'French, France'],
    u'fr-HT': [0x3c0c, u'French, Haiti'],
    u'fr-LU': [0x140c, u'French, Luxembourg'],
    u'fr-MA': [0x380c, u'French, Morocco'],
    u'fr-MC': [0x180c, u'French, Monaco'],
    u'fr-ML': [0x340c, u'French, Mali'],
    u'fr-RE': [0x200c, u'French, Réunion'],
    u'fr-SN': [0x280c, u'French, Senegal'],
    u'fr-West': [0x1c0c, u'French'],
    u'fuv-NG': [0x0467, u'Nigerian Fulfulde, Nigeria'],
    u'fy': [0x0062, u'Western Frisian'],
    u'fy-NL': [0x0462, u'Western Frisian, Netherlands'],
    u'ga': [0x003c, u'Irish'],
    u'ga-IE': [0x083c, u'Irish, Ireland'],
    u'gaz-ET': [0x0472, u'West Central Oromo, Ethiopia'],
    u'gd': [0x0091, u'Scottish Gaelic'],
    u'gd-GB': [0x0491, u'Scottish Gaelic, United Kingdom'],
    u'gl': [0x0056, u'Galician'],
    u'gl-ES': [0x0456, u'Galician, Spain'],
    u'gn-PY': [0x0474, u'Guarani, Paraguay'],
    u'gsw': [0x0084, u'Swiss German'],
    u'gsw-FR': [0x0484, u'Swiss German, France'],
    u'gu': [0x0047, u'Gujarati'],
    u'gu-IN': [0x0447, u'Gujarati, India'],
    u'ha': [0x0068, u'Hausa'],
    u'ha-Latn': [0x7c68, u'Hausa, Latin'],
    u'ha-Latn-NG': [0x0468, u'Hausa, Latin, Nigeria'],
    u'haw-US': [0x0475, u'Hawaiian, United States'],
    u'he': [0x000d, u'Hebrew'],
    u'he-IL': [0x040d, u'Hebrew, Israel'],
    u'hi': [0x0039, u'Hindi'],
    u'hi-IN': [0x0439, u'Hindi, India'],
    u'hr': [0x001a, u'Croatian'],
    u'hr-BA': [0x101a, u'Croatian, Bosnia and Herzegovina'],
    u'hr-HR': [0x041a, u'Croatian, Croatia'],
    u'hsb': [0x002e, u'Upper Sorbian'],
    u'hu': [0x000e, u'Hungarian'],
    u'hu-HU': [0x040e, u'Hungarian, Hungary'],
    u'hy': [0x002b, u'Armenian'],
    u'hy-AM': [0x042b, u'Armenian, Armenia'],
    u'ibb-NG': [0x0469, u'Ibibio, Nigeria'],
    u'id': [0x0021, u'Indonesian'],
    u'id-ID': [0x0421, u'Indonesian, Indonesia'],
    u'ig': [0x0070, u'Igbo'],
    u'ig-NG': [0x0470, u'Igbo, Nigeria'],
    u'ii': [0x0078, u'Sichuan Yi'],
    u'ii-CN': [0x0478, u'Sichuan Yi, China'],
    u'is': [0x000f, u'Icelandic'],
    u'is-IS': [0x040f, u'Icelandic, Iceland'],
    u'it': [0x0010, u'Italian'],
    u'it-CH': [0x0810, u'Italian, Switzerland'],
    u'it-IT': [0x0410, u'Italian, Italy'],
    u'iu': [0x005d, u'Inuktitut'],
    u'iu-Cans': [0x785d, u'Inuktitut, Unified Canadian Aboriginal Syllabics'],
    u'iu-Cans-CA': [
        0x045d, 'Inuktitut, Unified Canadian Aboriginal Syllabics, Canada'],
    u'iu-Latn': [0x7c5d, u'Inuktitut, Latin'],
    u'iu-Latn-CA': [0x085d, u'Inuktitut, Latin, Canada'],
    u'ja': [0x0011, u'Japanese'],
    u'ja-JP': [0x0411, u'Japanese, Japan'],
    u'ka': [0x0037, u'Georgian'],
    u'ka-GE': [0x0437, u'Georgian, Georgia'],
    u'kk': [0x003f, u'Kazakh'],
    u'kk-KZ': [0x043f, u'Kazakh, Kazakhstan'],
    u'kl': [0x006f, u'Kalaallisut'],
    u'kl-GL': [0x046f, u'Kalaallisut, Greenland'],
    u'km': [0x0053, u'Central Khmer'],
    u'km-KH': [0x0453, u'Central Khmer, Cambodia'],
    u'kn': [0x004b, u'Kannada'],
    u'kn-IN': [0x044b, u'Kannada, India'],
    u'ko': [0x0012, u'Korean'],
    u'kok': [0x0057, u'Konkani (macrolanguage)'],
    u'kok-IN': [0x0457, u'Konkani (macrolanguage), India'],
    u'ko-KR': [0x0412, u'Korean, Republic of Korea'],
    u'kr-NG': [0x0471, u'Kanuri, Nigeria'],
    u'ky': [0x0040, u'Kirghiz'],
    u'ky-KG': [0x0440, u'Kirghiz, Kyrgyzstan'],
    u'lb': [0x006e, u'Luxembourgish'],
    u'lb-LU': [0x046e, u'Luxembourgish, Luxembourg'],
    u'lo': [0x0054, u'Lao'],
    u'lo-LA': [0x0454, u'Lao, Lao People\'s Democratic Republic'],
    u'lt': [0x0027, u'Lithuanian'],
    u'lt-LT': [0x0427, u'Lithuanian, Lithuania'],
    u'lv': [0x0026, u'Latvian'],
    u'lv-LV': [0x0426, u'Latvian, Latvia'],
    u'mi': [0x0081, u'Maori'],
    u'mi-NZ': [0x0481, u'Maori, New Zealand'],
    u'mk': [0x002f, u'Macedonian'],
    u'mk-MK': [
        0x042f, u'Macedonian, The Former Yugoslav Republic of Macedonia'],
    u'ml': [0x004c, u'Malayalam'],
    u'ml-IN': [0x044c, u'Malayalam, India'],
    u'mn': [0x0050, u'Mongolian'],
    u'mn-Cyrl': [0x7850, u'Mongolian, Cyrillic'],
    u'mni': [0x0458, u'Manipuri'],
    u'mn-MN': [0x0450, u'Mongolian, Mongolia'],
    u'mn-Mong': [0x7c50, u'Mongolian, Mongolian'],
    u'mn-Mong-CN': [0x0850, u'Mongolian, Mongolian, China'],
    u'moh': [0x007c, u'Mohawk'],
    u'moh-CA': [0x047c, u'Mohawk, Canada'],
    u'mr': [0x004e, u'Marathi'],
    u'mr-IN': [0x044e, u'Marathi, India'],
    u'ms': [0x003e, u'Malay (macrolanguage)'],
    u'ms-BN': [0x083e, u'Malay (macrolanguage), Brunei Darussalam'],
    u'ms-MY': [0x043e, u'Malay (macrolanguage), Malaysia'],
    u'mt': [0x003a, u'Maltese'],
    u'mt-MT': [0x043a, u'Maltese, Malta'],
    u'my-MM': [0x0455, u'Burmese, Myanmar'],
    u'nb': [0x7c14, u'Norwegian Bokmål'],
    u'nb-NO': [0x0414, u'Norwegian Bokmål, Norway'],
    u'ne': [0x0061, u'Nepali'],
    u'ne-IN': [0x0861, u'Nepali, India'],
    u'ne-NP': [0x0461, u'Nepali, Nepal'],
    u'nl': [0x0013, u'Dutch'],
    u'nl-BE': [0x0813, u'Dutch, Belgium'],
    u'nl-NL': [0x0413, u'Dutch, Netherlands'],
    u'nn': [0x7814, u'Norwegian Nynorsk'],
    u'nn-NO': [0x0814, u'Norwegian Nynorsk, Norway'],
    u'no': [0x0014, u'Norwegian'],
    u'nso': [0x006c, u'Pedi'],
    u'nso-ZA': [0x046c, u'Pedi, South Africa'],
    u'oc': [0x0082, u'Occitan (post 1500)'],
    u'oc-FR': [0x0482, u'Occitan (post 1500), France'],
    u'or': [0x0048, u'Oriya'],
    u'or-IN': [0x0448, u'Oriya, India'],
    u'pa': [0x0046, u'Panjabi'],
    u'pa-IN': [0x0446, u'Panjabi, India'],
    u'pap-AN': [0x0479, u'Papiamento, Netherlands Antilles'],
    u'pa-PK': [0x0846, u'Panjabi, Pakistan'],
    u'pl': [0x0015, u'Polish'],
    u'pl-PL': [0x0415, u'Polish, Poland'],
    u'plt-MG': [0x048d, u'Plateau Malagasy, Madagascar'],
    u'prs': [0x008c, u'Dari'],
    u'prs-AF': [0x048c, u'Dari, Afghanistan'],
    u'ps': [0x0063, u'Pushto'],
    u'ps-AF': [0x0463, u'Pushto, Afghanistan'],
    u'pt': [0x0016, u'Portuguese'],
    u'pt-BR': [0x0416, u'Portuguese, Brazil'],
    u'pt-PT': [0x0816, u'Portuguese, Portugal'],
    u'qut': [0x0086, u'Guatemala'],
    u'qut-GT': [0x0486, u'Guatemala'],
    u'quz': [0x006b, u'Cusco Quechua'],
    u'quz-BO': [0x046b, u'Cusco Quechua, Bolivia'],
    u'quz-EC': [0x086b, u'Cusco Quechua, Ecuador'],
    u'quz-PE': [0x0c6b, u'Cusco Quechua, Peru'],
    u'rm': [0x0017, u'Romansh'],
    u'rm-CH': [0x0417, u'Romansh, Switzerland'],
    u'ro': [0x0018, u'Romanian'],
    u'ro-MO': [0x0818, u'Romanian, Macao'],
    u'ro-RO': [0x0418, u'Romanian, Romania'],
    u'ru': [0x0019, u'Russian'],
    u'ru-MO': [0x0819, u'Russian, Macao'],
    u'ru-RU': [0x0419, u'Russian, Russian Federation'],
    u'rw': [0x0087, u'Kinyarwanda'],
    u'rw-RW': [0x0487, u'Kinyarwanda, Rwanda'],
    u'sa': [0x004f, u'Sanskrit'],
    u'sah': [0x0085, u'Yakut'],
    u'sah-RU': [0x0485, u'Yakut, Russian Federation'],
    u'sa-IN': [0x044f, u'Sanskrit, India'],
    u'sd-IN': [0x0459, u'Sindhi, India'],
    u'sd-PK': [0x0859, u'Sindhi, Pakistan'],
    u'se': [0x003b, u'Northern Sami'],
    u'se-FI': [0x0c3b, u'Northern Sami, Finland'],
    u'se-NO': [0x043b, u'Northern Sami, Norway'],
    u'se-SE': [0x083b, u'Northern Sami, Sweden'],
    u'si': [0x005b, u'Sinhala'],
    u'si-LK': [0x045b, u'Sinhala, Sri Lanka'],
    u'sk': [0x001b, u'Slovak'],
    u'sk-SK': [0x041b, u'Slovak, Slovakia'],
    u'sl': [0x0024, u'Slovenian'],
    u'sl-SI': [0x0424, u'Slovenian, Slovenia'],
    u'sma': [0x783b, u'Southern Sami'],
    u'sma-NO': [0x183b, u'Southern Sami, Norway'],
    u'sma-SE': [0x1c3b, u'Southern Sami, Sweden'],
    u'smj': [0x7c3b, u'Lule Sami'],
    u'smj-NO': [0x103b, u'Lule Sami, Norway'],
    u'smj-SE': [0x143b, u'Lule Sami, Sweden'],
    u'smn': [0x703b, u'Inari Sami'],
    u'smn-FI': [0x243b, u'Inari Sami, Finland'],
    u'sms': [0x743b, u'Skolt Sami'],
    u'sms-FI': [0x203b, u'Skolt Sami, Finland'],
    u'so-SO': [0x0477, u'Somali, Somalia'],
    u'sq': [0x001c, u'Albanian'],
    u'sq-AL': [0x041c, u'Albanian, Albania'],
    u'sr': [0x7c1a, u'Serbian'],
    u'sr-Cyrl': [0x6c1a, u'Serbian, Cyrillic'],
    u'sr-Cyrl-BA': [0x1c1a, u'Serbian, Cyrillic, Bosnia and Herzegovina'],
    u'sr-Cyrl-CS': [0x0c1a, u'Serbian, Cyrillic, Serbia and Montenegro'],
    u'sr-Cyrl-ME': [0x301a, u'Serbian, Cyrillic, Montenegro'],
    u'sr-Cyrl-RS': [0x281a, u'Serbian, Cyrillic, Serbia'],
    u'sr-Latn': [0x701a, u'Serbian, Latin'],
    u'sr-Latn-BA': [0x181a, u'Serbian, Latin, Bosnia and Herzegovina'],
    u'sr-Latn-CS': [0x081a, u'Serbian, Latin, Serbia and Montenegro'],
    u'sr-Latn-ME': [0x2c1a, u'Serbian, Latin, Montenegro'],
    u'sr-Latn-RS': [0x241a, u'Serbian, Latin, Serbia'],
    u'st-ZA': [0x0430, u'Southern Sotho, South Africa'],
    u'sv': [0x001d, u'Swedish'],
    u'sv-FI': [0x081d, u'Swedish, Finland'],
    u'sv-SE': [0x041d, u'Swedish, Sweden'],
    u'sw': [0x0041, u'Swahili (macrolanguage)'],
    u'sw-KE': [0x0441, u'Swahili (macrolanguage), Kenya'],
    u'syr': [0x005a, u'Syriac'],
    u'syr-SY': [0x045a, u'Syriac, Syrian Arab Republic'],
    u'ta': [0x0049, u'Tamil'],
    u'ta-IN': [0x0449, u'Tamil, India'],
    u'te': [0x004a, u'Telugu'],
    u'te-IN': [0x044a, u'Telugu, India'],
    u'tg': [0x0028, u'Tajik'],
    u'tg-Cyrl': [0x7c28, u'Tajik, Cyrillic'],
    u'tg-Cyrl-TJ': [0x0428, u'Tajik, Cyrillic, Tajikistan'],
    u'th': [0x001e, u'Thai'],
    u'th-TH': [0x041e, u'Thai, Thailand'],
    u'ti-ER': [0x0473, u'Tigrinya, Eritrea'],
    u'ti-ET': [0x0873, u'Tigrinya, Ethiopia'],
    u'tk': [0x0042, u'Turkmen'],
    u'tk-TM': [0x0442, u'Turkmen, Turkmenistan'],
    u'tmz': [0x045f, u'Tamanaku'],
    u'tmz-MA': [0x0c5f, u'Tamanaku, Morocco'],
    u'tn': [0x0032, u'Tswana'],
    u'tn-ZA': [0x0432, u'Tswana, South Africa'],
    u'tr': [0x001f, u'Turkish'],
    u'tr-TR': [0x041f, u'Turkish, Turkey'],
    u'ts-ZA': [0x0431, u'Tsonga, South Africa'],
    u'tt': [0x0044, u'Tatar'],
    u'tt-RU': [0x0444, u'Tatar, Russian Federation'],
    u'tzm': [0x005f, u'Central Atlas Tamazight'],
    u'tzm-Latn': [0x7c5f, u'Central Atlas Tamazight, Latin'],
    u'tzm-Latn-DZ': [0x085f, u'Central Atlas Tamazight, Latin, Algeria'],
    u'ug': [0x0080, u'Uighur'],
    u'ug-CN': [0x0480, u'Uighur, China'],
    u'uk': [0x0022, u'Ukrainian'],
    u'uk-UA': [0x0422, u'Ukrainian, Ukraine'],
    u'ur': [0x0020, u'Urdu'],
    u'ur-IN': [0x0820, u'Urdu, India'],
    u'ur-PK': [0x0420, u'Urdu, Pakistan'],
    u'uz': [0x0043, u'Uzbek'],
    u'uz-Cyrl': [0x7843, u'Uzbek, Cyrillic'],
    u'uz-Cyrl-UZ': [0x0843, u'Uzbek, Cyrillic, Uzbekistan'],
    u'uz-Latn': [0x7c43, u'Uzbek, Latin'],
    u'uz-Latn-UZ': [0x0443, u'Uzbek, Latin, Uzbekistan'],
    u'ven-ZA': [0x0433, u'South Africa'],
    u'vi': [0x002a, u'Vietnamese'],
    u'vi-VN': [0x042a, u'Vietnamese, Viet Nam'],
    u'wen-DE': [0x042e, u'Sorbian languages, Germany'],
    u'wo': [0x0088, u'Wolof'],
    u'wo-SN': [0x0488, u'Wolof, Senegal'],
    u'xh': [0x0034, u'Xhosa'],
    u'xh-ZA': [0x0434, u'Xhosa, South Africa'],
    u'yo': [0x006a, u'Yoruba'],
    u'yo-NG': [0x046a, u'Yoruba, Nigeria'],
    u'zh': [0x7804, u'Chinese'],
    u'zh-CN': [0x0804, u'Chinese, China'],
    u'zh-Hans': [0x0004, u'Chinese, Han (Simplified variant)'],
    u'zh-Hant': [0x7c04, u'Chinese, Han (Traditional variant)'],
    u'zh-HK': [0x0c04, u'Chinese, Hong Kong'],
    u'zh-MO': [0x1404, u'Chinese, Macao'],
    u'zh-SG': [0x1004, u'Chinese, Singapore'],
    u'zh-TW': [0x0404, u'Chinese, Taiwan, Province of China'],
    u'zu': [0x0035, u'Zulu'],
    u'zu-ZA': [0x0435, u'Zulu, South Africa'],
}

# Make sure the language identifiers are lower case.
LANGUAGE_IDENTIFIERS = dict(
    (key.lower(), value) for key, value in LANGUAGE_IDENTIFIERS.items())



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This file contains a unit test for the timelib in Plaso."""

import datetime
import unittest

from plaso.lib import errors
from plaso.lib import timelib

import pytz  # pylint: disable=wrong-import-order


class TimeLibTest(unittest.TestCase):
  """Tests for timestamp."""

  def testCopyFromString(self):
    """Tests the CopyFromString function."""
    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27')
    expected_timestamp = 1340755200000000
    self.assertEqual(timestamp, expected_timestamp)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(None)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-6-27')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-00-27')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-13-27')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-01-00')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-01-32')

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01')
    expected_timestamp = 1340821021000000
    self.assertEqual(timestamp, expected_timestamp)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:1')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27T18:17:01')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 24:17:01')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:60:01')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:60')

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.123')
    expected_timestamp = 1340821021123000
    self.assertEqual(timestamp, expected_timestamp)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.12')

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.123456')
    expected_timestamp = 1340821021123456
    self.assertEqual(timestamp, expected_timestamp)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.1234')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01.1234567')

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+00:00')
    expected_timestamp = 1340821021000000
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+01:00')
    expected_timestamp = 1340817421000000
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01-07:00')
    expected_timestamp = 1340846221000000
    self.assertEqual(timestamp, expected_timestamp)

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+1')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+01')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+01:0')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01+00:00:0')

    with self.assertRaises(ValueError):
      _ = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01Z')

  def testTimestampIsLeapYear(self):
    """Tests the is leap year check."""
    self.assertEqual(timelib.Timestamp.IsLeapYear(2012), True)
    self.assertEqual(timelib.Timestamp.IsLeapYear(2013), False)
    self.assertEqual(timelib.Timestamp.IsLeapYear(2000), True)
    self.assertEqual(timelib.Timestamp.IsLeapYear(1900), False)

  def testTimestampDaysInMonth(self):
    """Tests the days in month function."""
    self.assertEqual(timelib.Timestamp.DaysInMonth(0, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(1, 2013), 28)
    self.assertEqual(timelib.Timestamp.DaysInMonth(1, 2012), 29)
    self.assertEqual(timelib.Timestamp.DaysInMonth(2, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(3, 2013), 30)
    self.assertEqual(timelib.Timestamp.DaysInMonth(4, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(5, 2013), 30)
    self.assertEqual(timelib.Timestamp.DaysInMonth(6, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(7, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(8, 2013), 30)
    self.assertEqual(timelib.Timestamp.DaysInMonth(9, 2013), 31)
    self.assertEqual(timelib.Timestamp.DaysInMonth(10, 2013), 30)
    self.assertEqual(timelib.Timestamp.DaysInMonth(11, 2013), 31)

    with self.assertRaises(ValueError):
      timelib.Timestamp.DaysInMonth(-1, 2013)

    with self.assertRaises(ValueError):
      timelib.Timestamp.DaysInMonth(12, 2013)

  def testTimestampDaysInYear(self):
    """Test the days in year function."""
    self.assertEqual(timelib.Timestamp.DaysInYear(2013), 365)
    self.assertEqual(timelib.Timestamp.DaysInYear(2012), 366)

  def testTimestampDayOfYear(self):
    """Test the day of year function."""
    self.assertEqual(timelib.Timestamp.DayOfYear(0, 0, 2013), 0)
    self.assertEqual(timelib.Timestamp.DayOfYear(0, 2, 2013), 31 + 28)
    self.assertEqual(timelib.Timestamp.DayOfYear(0, 2, 2012), 31 + 29)

    expected_day_of_year = 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30
    self.assertEqual(
        timelib.Timestamp.DayOfYear(0, 11, 2013), expected_day_of_year)

  def testTimestampFromPosixTime(self):
    """Test the POSIX time conversion."""
    timestamp = timelib.Timestamp.FromPosixTime(1281647191)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-08-12 21:06:31')
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.FromPosixTime(-122557518)
    expected_timestamp = timelib.Timestamp.FromTimeString(
        u'1966-02-12 1966 12:14:42 UTC')
    self.assertEqual(timestamp, expected_timestamp)

    # POSIX time that exceeds upper bound.
    self.assertEqual(timelib.Timestamp.FromPosixTime(9223372036855), 0)

    # POSIX time that exceeds lower bound.
    self.assertEqual(timelib.Timestamp.FromPosixTime(-9223372036855), 0)

  def testMonthDict(self):
    """Test the month dict, both inside and outside of scope."""
    self.assertEqual(timelib.MONTH_DICT[u'nov'], 11)
    self.assertEqual(timelib.MONTH_DICT[u'jan'], 1)
    self.assertEqual(timelib.MONTH_DICT[u'may'], 5)

    month = timelib.MONTH_DICT.get(u'doesnotexist')
    self.assertIsNone(month)

  def testLocaltimeToUTC(self):
    """Test the localtime to UTC conversion."""
    timezone = pytz.timezone(u'CET')

    local_timestamp = timelib.Timestamp.CopyFromString(u'2013-01-01 01:00:00')
    timestamp = timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-01-01 00:00:00')
    self.assertEqual(timestamp, expected_timestamp)

    local_timestamp = timelib.Timestamp.CopyFromString(u'2013-07-01 02:00:00')
    timestamp = timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-01 00:00:00')
    self.assertEqual(timestamp, expected_timestamp)

    # In the local timezone this is a non-existent timestamp.
    local_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-03-31 02:00:00')
    with self.assertRaises(pytz.NonExistentTimeError):
      timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone, is_dst=None)

    timestamp = timelib.Timestamp.LocaltimeToUTC(
        local_timestamp, timezone, is_dst=True)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-03-31 00:00:00')
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.LocaltimeToUTC(
        local_timestamp, timezone, is_dst=False)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-03-31 01:00:00')
    self.assertEqual(timestamp, expected_timestamp)

    # In the local timezone this is an ambiguous timestamp.
    local_timestamp = timelib.Timestamp.CopyFromString(u'2013-10-27 02:30:00')

    with self.assertRaises(pytz.AmbiguousTimeError):
      timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone, is_dst=None)

    timestamp = timelib.Timestamp.LocaltimeToUTC(
        local_timestamp, timezone, is_dst=True)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-27 00:30:00')
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-27 01:30:00')
    self.assertEqual(timestamp, expected_timestamp)

    # Use the UTC timezone.
    self.assertEqual(
        timelib.Timestamp.LocaltimeToUTC(local_timestamp, pytz.UTC),
        local_timestamp)

    # Use a timezone in the Western Hemisphere.
    timezone = pytz.timezone(u'EST')

    local_timestamp = timelib.Timestamp.CopyFromString(u'2013-01-01 00:00:00')
    timestamp = timelib.Timestamp.LocaltimeToUTC(local_timestamp, timezone)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-01-01 05:00:00')
    self.assertEqual(timestamp, expected_timestamp)

  def testCopyToDatetime(self):
    """Test the copy to datetime object."""
    timezone = pytz.timezone(u'CET')

    timestamp = timelib.Timestamp.CopyFromString(u'2013-03-14 20:20:08.850041')
    datetime_object = timelib.Timestamp.CopyToDatetime(timestamp, timezone)
    expected_datetime_object = datetime.datetime(
        2013, 3, 14, 21, 20, 8, 850041, tzinfo=timezone)
    self.assertEqual(datetime_object, expected_datetime_object)

  def testCopyToPosix(self):
    """Test converting microseconds to seconds."""
    timestamp = timelib.Timestamp.CopyFromString(u'2013-10-01 12:00:00')
    expected_posixtime, _ = divmod(timestamp, 1000000)
    posixtime = timelib.Timestamp.CopyToPosix(timestamp)
    self.assertEqual(posixtime, expected_posixtime)

  def testTimestampFromTimeString(self):
    """The the FromTimeString function."""
    # Test daylight savings.
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-01 12:00:00')

    # Check certain variance of this timestamp.
    timestamp = timelib.Timestamp.FromTimeString(
        u'2013-10-01 14:00:00', timezone=pytz.timezone(u'Europe/Rome'))
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.FromTimeString(
        u'2013-10-01 12:00:00', timezone=pytz.timezone(u'UTC'))
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.FromTimeString(
        u'2013-10-01 05:00:00', timezone=pytz.timezone(u'PST8PDT'))
    self.assertEqual(timestamp, expected_timestamp)

    # Now to test outside of the daylight savings.
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-02-01 12:00:00')

    timestamp = timelib.Timestamp.FromTimeString(
        u'2014-02-01 13:00:00', timezone=pytz.timezone(u'Europe/Rome'))
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.FromTimeString(
        u'2014-02-01 12:00:00', timezone=pytz.timezone(u'UTC'))
    self.assertEqual(timestamp, expected_timestamp)

    timestamp = timelib.Timestamp.FromTimeString(
        u'2014-02-01 04:00:00', timezone=pytz.timezone(u'PST8PDT'))
    self.assertEqual(timestamp, expected_timestamp)

    # Define two timestamps, one being GMT and the other UTC.
    time_string_utc = u'Wed 05 May 2010 03:52:31 UTC'
    time_string_gmt = u'Wed 05 May 2010 03:52:31 GMT'

    timestamp_utc = timelib.Timestamp.FromTimeString(time_string_utc)
    timestamp_gmt = timelib.Timestamp.FromTimeString(time_string_gmt)

    # Test if these two are different, and if so, then we'll try again
    # using the 'gmt_is_utc' flag, which then should result to the same
    # results.
    if timestamp_utc != timestamp_gmt:
      self.assertEqual(timestamp_utc, timelib.Timestamp.FromTimeString(
          time_string_gmt, gmt_as_timezone=False))

    timestamp = timelib.Timestamp.FromTimeString(
        u'12-15-1984 05:13:00', timezone=pytz.timezone(u'EST5EDT'))
    self.assertEqual(timestamp, 471953580000000)

    # Swap day and month.
    timestamp = timelib.Timestamp.FromTimeString(
        u'12-10-1984 05:13:00', timezone=pytz.timezone(u'EST5EDT'),
        dayfirst=True)
    self.assertEqual(timestamp, 466420380000000)

    timestamp = timelib.Timestamp.FromTimeString(u'12-15-1984 10:13:00Z')
    self.assertEqual(timestamp, 471953580000000)

    # Setting the timezone for string that already contains a timezone
    # indicator should not affect the conversion.
    timestamp = timelib.Timestamp.FromTimeString(
        u'12-15-1984 10:13:00Z', timezone=pytz.timezone(u'EST5EDT'))
    self.assertEqual(timestamp, 471953580000000)

    timestamp = timelib.Timestamp.FromTimeString(u'15/12/1984 10:13:00Z')
    self.assertEqual(timestamp, 471953580000000)

    timestamp = timelib.Timestamp.FromTimeString(u'15-12-84 10:13:00Z')
    self.assertEqual(timestamp, 471953580000000)

    timestamp = timelib.Timestamp.FromTimeString(
        u'15-12-84 10:13:00-04', timezone=pytz.timezone(u'EST5EDT'))
    self.assertEqual(timestamp, 471967980000000)

    with self.assertRaises(errors.TimestampError):
      timestamp = timelib.Timestamp.FromTimeString(
          u'thisisnotadatetime', timezone=pytz.timezone(u'EST5EDT'))

    timestamp = timelib.Timestamp.FromTimeString(
        u'12-15-1984 04:13:00', timezone=pytz.timezone(u'America/Chicago'))
    self.assertEqual(timestamp, 471953580000000)

    timestamp = timelib.Timestamp.FromTimeString(
        u'07-14-1984 23:13:00', timezone=pytz.timezone(u'America/Chicago'))
    self.assertEqual(timestamp, 458712780000000)

    timestamp = timelib.Timestamp.FromTimeString(
        u'12-15-1984 05:13:00', timezone=pytz.timezone(u'US/Pacific'))
    self.assertEqual(timestamp, 471964380000000)

  def testRoundTimestamp(self):
    """Test the RoundToSeconds function."""
    # Should be rounded up.
    test_one = 442813351785412
    # Should be rounded down.
    test_two = 1384381247271976

    self.assertEqual(
        timelib.Timestamp.RoundToSeconds(test_one), 442813352000000)
    self.assertEqual(
        timelib.Timestamp.RoundToSeconds(test_two), 1384381247000000)

  def testTimestampFromTimeParts(self):
    """Test the FromTimeParts function."""
    timestamp = timelib.Timestamp.FromTimeParts(
        2013, 6, 25, 22, 19, 46, 0, timezone=pytz.timezone(u'PST8PDT'))
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-06-25 22:19:46-07:00')
    self.assertEqual(timestamp, expected_timestamp)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-06-26 05:19:46')
    timestamp = timelib.Timestamp.FromTimeParts(2013, 6, 26, 5, 19, 46)
    self.assertEqual(timestamp, expected_timestamp)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-06-26 05:19:46.000542')
    timestamp = timelib.Timestamp.FromTimeParts(
        2013, 6, 26, 5, 19, 46, microseconds=542)
    self.assertEqual(timestamp, expected_timestamp)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome cookie database plugin."""

import unittest

from plaso.formatters import chrome_cookies  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers.sqlite_plugins import chrome_cookies

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class ChromeCookiesPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome cookie database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'cookies.db'])
  def testProcess(self):
    """Tests the Process function on a Chrome cookie database file."""
    plugin = chrome_cookies.ChromeCookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'cookies.db'], plugin)

    # Since we've got both events generated by cookie plugins and the Chrome
    # cookie plugin we need to separate them.
    events = []
    extra_objects = []

    for event in storage_writer.GetEvents():
      if event.data_type == u'chrome:cookie:entry':
        events.append(event)
      else:
        extra_objects.append(event)

    # The cookie database contains 560 entries:
    #     560 creation timestamps.
    #     560 last access timestamps.
    #     560 expired timestamps.
    # Then there are extra events created by plugins:
    #      75 events created by Google Analytics cookies.
    # In total: 1755 events.
    self.assertEqual(len(events), 3 * 560)
    self.assertEqual(len(extra_objects), 75)

    # Check few "random" events to verify.

    # Check one linkedin cookie.
    event = events[124]
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_ACCESS)
    self.assertEqual(event.host, u'www.linkedin.com')
    self.assertEqual(event.cookie_name, u'leo_auth_token')
    self.assertFalse(event.httponly)
    self.assertEqual(event.url, u'http://www.linkedin.com/')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2011-08-25 21:50:27.292367')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'http://www.linkedin.com/ (leo_auth_token) Flags: [HTTP only] = False '
        u'[Persistent] = True')
    expected_short_message = u'www.linkedin.com (leo_auth_token)'
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Check one of the visits to rubiconproject.com.
    event = events[379]
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_LAST_ACCESS)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-01 13:54:34.949210')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.url, u'http://rubiconproject.com/')
    self.assertEqual(event.path, u'/')
    self.assertFalse(event.secure)
    self.assertTrue(event.persistent)

    expected_message = (
        u'http://rubiconproject.com/ (put_2249) Flags: [HTTP only] = False '
        u'[Persistent] = True')
    self._TestGetMessageStrings(
        event, expected_message, u'rubiconproject.com (put_2249)')

    # Examine an event for a visit to a political blog site.
    event = events[444]
    self.assertEqual(
        event.path,
        u'/2012/03/21/romney-tries-to-clean-up-etch-a-sketch-mess/')
    self.assertEqual(event.host, u'politicalticker.blogs.cnn.com')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-22 01:47:21.012022')
    self.assertEqual(event.timestamp, expected_timestamp)

    # Examine a cookie that has an autologin entry.
    event = events[1425]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-01 13:52:56.189444')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.host, u'marvel.com')
    self.assertEqual(event.cookie_name, u'autologin[timeout]')
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)
    # This particular cookie value represents a timeout value that corresponds
    # to the expiration date of the cookie.
    self.assertEqual(event.data, u'1364824322')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The UserAssist Windows Registry event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class UserAssistWindowsRegistryEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for an UserAssist Windows Registry event."""

  DATA_TYPE = u'windows:registry:userassist'

  FORMAT_STRING_PIECES = [
      u'[{key_path}]',
      u'UserAssist entry: {entry_index}',
      u'Value name: {value_name}',
      u'Count: {number_of_executions}',
      u'Application focus count: {application_focus_count}',
      u'Application focus duration: {application_focus_duration}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{value_name}',
      u'Count: {number_of_executions}']

  SOURCE_LONG = u'Registry Key: UserAssist'
  SOURCE_SHORT = u'REG'


manager.FormattersManager.RegisterFormatter(
    UserAssistWindowsRegistryEventFormatter)



# -*- coding: utf-8 -*-
"""The Mac OS X keychain password database file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class KeychainApplicationRecordFormatter(interface.ConditionalEventFormatter):
  """Formatter for a keychain application record event."""

  DATA_TYPE = u'mac:keychain:application'

  FORMAT_STRING_PIECES = [
      u'Name: {entry_name}',
      u'Account: {account_name}']

  FORMAT_STRING_SHORT_PIECES = [u'{entry_name}']

  SOURCE_LONG = u'Keychain Application password'
  SOURCE_SHORT = u'LOG'


class KeychainInternetRecordFormatter(interface.ConditionalEventFormatter):
  """Formatter for a keychain Internet record event."""

  DATA_TYPE = u'mac:keychain:internet'

  FORMAT_STRING_PIECES = [
      u'Name: {entry_name}',
      u'Account: {account_name}',
      u'Where: {where}',
      u'Protocol: {protocol}',
      u'({type_protocol})']

  FORMAT_STRING_SHORT_PIECES = [u'{entry_name}']

  SOURCE_LONG = u'Keychain Internet password'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatters([
    KeychainApplicationRecordFormatter, KeychainInternetRecordFormatter])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Viper analysis plugin."""

import unittest

try:
  import mock  # pylint: disable=import-error
except ImportError:
  from unittest import mock

from dfdatetime import posix_time as dfdatetime_posix_time
from dfvfs.path import fake_path_spec

from plaso.analysis import viper
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib

from tests.analysis import test_lib


class MockResponse(dict):
  """A mock object to simulate a response object from the requests library."""

  def json(self):
    """Provided for compatibility with the requests library."""
    return self

  def raise_for_status(self):
    """Provided for compatibility with the requests library."""
    return


class ViperTest(test_lib.AnalysisPluginTestCase):
  """Tests for the Viper analysis plugin."""

  _EVENT_1_HASH = (
      u'2d79fcc6b02a2e183a0cb30e0e25d103f42badda9fbf86bbee06f93aa3855aff')

  _TEST_EVENTS = [{
      u'timestamp': timelib.Timestamp.CopyFromString(u'2015-01-01 17:00:00'),
      u'sha256_hash': _EVENT_1_HASH}]

  def _MockPost(self, unused_url, data=None):
    """Mock funtion to simulate a Viper API request.

    Args:
      url (str): URL being requested.
      data (dict[str, object]): simulated form data for the Viper API request.

    Returns:
      MockResponse: mocked response that simulates a real response object
          returned by the requests library from the Viper API.
    """
    sha256_hash = data.get(u'sha256', None)
    if sha256_hash != self._EVENT_1_HASH:
      self.fail(u'Unexpected data in request.post().')

    response = MockResponse()
    response[u'default'] = ({
        u'sha1': u'13da502ab0d75daca5e5075c60e81bfe3b7a637f',
        u'name': u'darkcomet.exe',
        u'tags': [
            u'rat',
            u'darkcomet'],
        u'sha512': u'7e81e0c4f49f1884ebebdf6e53531e7836721c2ae417'
                   u'29cf5bc0340f3369e7d37fe4168a7434b2b0420b299f5c'
                   u'1d9a4f482f1bda8e66e40345757d97e5602b2d',
        u'created_at': u'2015-03-30 23:13:20.595238',
        u'crc32': u'2238B48E',
        u'ssdeep': u'12288:D9HFJ9rJxRX1uVVjoaWSoynxdO1FVBaOiRZTERfIhNk'
                   u'NCCLo9Ek5C/hlg:NZ1xuVVjfFoynPaVBUR8f+kN10EB/g',
        u'sha256': u'2d79fcc6b02a2e183a0cb30e0e25d103f42badda9fbf86bbee06f9'
                   u'3aa3855aff',
        u'type': u'PE32 executable (GUI) Intel 80386, for MS Windows',
        u'id': 10,
        u'md5': u'9f2520a3056543d49bb0f822d85ce5dd',
        u'size': 774144},)

    return response

  def _CreateTestEventObject(self, event_dictionary):
    """Create a test event with a set of attributes.

    Args:
      event_dictionary (dict[str, str]): contains attributes of an event to add
          to the queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    date_time = dfdatetime_posix_time.PosixTime(
        timestamp=event_dictionary[u'timestamp'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)

    event.data_type = u'pe:compilation:compilation_time'
    event.pe_type = u'Executable (EXE)'

    for attribute_name, attribute_value in event_dictionary.items():
      if attribute_name == u'timestamp':
        continue

      setattr(event, attribute_name, attribute_value)

    return event

  def setUp(self):
    """Makes preparations before running an individual test."""
    self.requests_patcher = mock.patch(u'requests.post', self._MockPost)
    self.requests_patcher.start()

  def tearDown(self):
    """Cleans up after running an individual test."""
    self.requests_patcher.stop()

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    events = []
    for event_dictionary in self._TEST_EVENTS:
      event_dictionary[u'pathspec'] = fake_path_spec.FakePathSpec(
          location=u'C:\\WINDOWS\\system32\\evil.exe')

      event = self._CreateTestEventObject(event_dictionary)
      events.append(event)

    plugin = viper.ViperAnalysisPlugin()
    plugin.SetHost(u'localhost')
    plugin.SetPort(8080)
    plugin.SetProtocol(u'http')

    storage_writer = self._AnalyzeEvents(events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(storage_writer.number_of_event_tags, 1)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = (
        u'viper hash tagging results\n'
        u'1 path specifications tagged with label: viper_present\n'
        u'1 path specifications tagged with label: viper_project_default\n'
        u'1 path specifications tagged with label: viper_tag_darkcomet\n'
        u'1 path specifications tagged with label: viper_tag_rat\n')

    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.GetEventTags():
      labels.extend(event_tag.labels)
    self.assertEqual(len(labels), 4)

    expected_labels = [
        u'viper_present', u'viper_project_default', u'viper_tag_darkcomet',
        u'viper_tag_rat']
    self.assertEqual(sorted(labels), expected_labels)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the output module interface classes."""

import abc
import logging

from plaso.lib import errors


class OutputModule(object):
  """Class that implements the output module object interface."""

  NAME = u''
  DESCRIPTION = u''

  def __init__(self, output_mediator):
    """Initializes an output module.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.

    Raises:
      ValueError: when there are unused keyword arguments.
    """
    super(OutputModule, self).__init__()
    self._output_mediator = output_mediator

  def _ReportEventError(self, event, error_message):
    """Reports an event related error.

    Args:
      event (EventObject): event.
      error_message (str): error message.
    """
    event_identifier = event.GetIdentifier()
    event_identifier_string = event_identifier.CopyToString()
    error_message = (
        u'Event: {0!s} data type: {1:s} display name: {2:s} '
        u'parser chain: {3:s} with error: {4:s}').format(
            event_identifier_string, event.data_type, event.display_name,
            event.parser, error_message)
    logging.error(error_message)

  def Close(self):
    """Closes the output."""
    pass

  def GetMissingArguments(self):
    """Retrieves arguments required by the module that have not been specified.

    Returns:
      list[str]: names of argument that are required by the module and have
          not been specified.
    """
    return []

  def Open(self):
    """Opens the output."""
    pass

  def WriteEvent(self, event):
    """Writes the event to the output.

    Args:
      event (EventObject): event.
    """
    self.WriteEventStart()

    try:
      self.WriteEventBody(event)

    except errors.NoFormatterFound as exception:
      error_message = u'unable to retrieve formatter with error: {0!s}'.format(
          exception)
      self._ReportEventError(event, error_message)

    except errors.WrongFormatter as exception:
      error_message = u'wrong formatter with error: {0!s}'.format(exception)
      self._ReportEventError(event, error_message)

    self.WriteEventEnd()

  @abc.abstractmethod
  def WriteEventBody(self, event):
    """Writes the body of an event to the output.

    Args:
      event (EventObject): event.
    """

  def WriteEventEnd(self):
    """Writes the end of an event to the output.

    Can be used for post-processing or output after an individual event
    has been written, such as writing closing XML tags, etc.
    """
    pass

  def WriteEventMACBGroup(self, event_macb_group):
    """Writes an event MACB group to the output.

    An event MACB group is a group of events that have the same timestamp and
    event data (attributes and values), where the timestamp description (or
    usage) is one or more of MACB (modification, access, change, birth).

    This function is called if the psort engine detected an event MACB group
    so that the output module, if supported, can represent the group as
    such. If not overridden this function will output every event individually.

    Args:
      event_macb_group (list[EventObject]): group of events with identical
          timestamps, attributes and values.
    """
    for event in event_macb_group:
      self.WriteEvent(event)

  def WriteEventStart(self):
    """Writes the start of an event to the output.

    Can be used for pre-processing or output before an individual event
    has been written, such as writing opening XML tags, etc.
    """
    pass

  def WriteFooter(self):
    """Writes the footer to the output.

    Can be used for post-processing or output after the last event
    is written, such as writing a file footer.
    """
    pass

  def WriteHeader(self):
    """Writes the header to the output.

    Can be used for pre-processing or output before the first event
    is written, such as writing a file header.
    """
    pass


class LinearOutputModule(OutputModule):
  """Class that implements a linear output module object."""

  # Need to suppress this since these classes do not implement the
  # abstract method WriteEventBody, classes that inherit from one of these
  # classes need to implement that function.
  # pylint: disable=abstract-method

  def __init__(self, output_mediator):
    """Initializes the output module object.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfvfs.

    Raises:
      ValueError: if the output writer is missing.
    """
    super(LinearOutputModule, self).__init__(output_mediator)
    self._output_writer = None

  def _WriteLine(self, line):
    """Write a single line to the supplied file-like object.

    Args:
      line (str): line of text to write.
    """
    self._output_writer.Write(line)

  def SetOutputWriter(self, output_writer):
    """Set the output writer.

    Args:
      output_writer (CLIOutputWriter): output writer.
    """
    self._output_writer = output_writer

  def Close(self):
    """Closes the output."""
    self._output_writer = None



# -*- coding: utf-8 -*-
"""The iMessage chat.db (OSX) and sms.db (iOS)database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class IMessageFormatter(interface.ConditionalEventFormatter):
  """Formatter for an iMessage and SMS event."""

  DATA_TYPE = u'imessage:event:chat'

  FORMAT_STRING_PIECES = [
      u'Row ID: {identifier}',
      u'iMessage ID: {imessage_id}',
      u'Read Receipt: {read_receipt}',
      u'Message Type: {message_type}',
      u'Service: {service}',
      u'Attachment Location: {attachment_location}',
      u'Message Content: {text}']

  FORMAT_STRING_SHORT_PIECES = [u'{text}']

  SOURCE_LONG = u'Apple iMessage Application'
  SOURCE_SHORT = u'iMessage'

  _READ_RECEIPT = {
      0: False,
      1: True
  }
  _MESSAGE_TYPE = {
      0: u'Received',
      1: u'Sent'
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    read_receipt = event_values.get(u'read_receipt', None)
    if read_receipt is not None:
      event_values[u'read_receipt'] = (
          self._READ_RECEIPT.get(read_receipt, u'UNKNOWN'))

    message_type = event_values.get(u'message_type', None)
    if message_type is not None:
      event_values[u'message_type'] = (
          self._MESSAGE_TYPE.get(message_type, u'UNKNOWN'))

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(IMessageFormatter)



# -*- coding: utf-8 -*-
"""Parser for Zsh extended_history files.

The file format is described here:
http://zsh.sourceforge.net/Doc/Release/Options.html#index-EXTENDEDHISTORY
"""
import re

import pyparsing

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.parsers import manager
from plaso.parsers import text_parser


class ZshHistoryEventData(events.EventData):
  """Zsh history event data.

  Attributes:
    command (str): command that was run.
    elapsed_seconds (int): number of seconds that the command took to execute.
  """
  DATA_TYPE = u'shell:zsh:history'

  def __init__(self):
    """Initializes event data."""
    super(ZshHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.command = None
    self.elapsed_seconds = None


class ZshExtendedHistoryParser(text_parser.PyparsingMultiLineTextParser):
  """Parser for Zsh extended_history files"""

  NAME = u'zsh_extended_history'
  DESCRIPTION = u'Parser for ZSH extended history files'

  _VERIFICATION_REGEX = re.compile(r'^:\s\d+:\d+;')

  _PYPARSING_COMPONENTS = {
      u'timestamp': text_parser.PyparsingConstants.INTEGER.
                    setResultsName(u'timestamp'),
      u'elapsed_seconds': text_parser.PyparsingConstants.INTEGER.
                          setResultsName(u'elapsed_seconds'),
      u'command': pyparsing.Regex(r'.+?(?=($|\n:\s\d+:\d+;))', re.DOTALL).
                  setResultsName(u'command'),
  }

  _LINE_GRAMMAR = (
      pyparsing.Literal(u':') +
      _PYPARSING_COMPONENTS[u'timestamp'] + pyparsing.Literal(u':') +
      _PYPARSING_COMPONENTS[u'elapsed_seconds'] + pyparsing.Literal(u';') +
      _PYPARSING_COMPONENTS[u'command'] + pyparsing.LineEnd())

  LINE_STRUCTURES = [(u'command', _LINE_GRAMMAR)]

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses a record and produces a Zsh history event.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): structure parsed from the log file.

    Raises:
      ParseError: when the structure type is unknown.
    """
    if key != u'command':
      raise errors.ParseError(
          u'Unable to parse record, unknown structure: {0:s}'.format(key))

    event_data = ZshHistoryEventData()
    event_data.command = structure[u'command']
    event_data.elapsed_seconds = structure[u'elapsed_seconds']

    date_time = dfdatetime_posix_time.PosixTime(
        timestamp=structure[u'timestamp'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_MODIFICATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, parser_mediator, lines):
    """Verifies whether content corresponds to a Zsh extended_history file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      line (str): single line from the text file.

    Returns:
      bool: True if the line was successfully parsed.
    """
    if self._VERIFICATION_REGEX.match(lines):
      return True


manager.ParsersManager.RegisterParser(ZshExtendedHistoryParser)



# -*- coding: utf-8 -*-
"""The server configuration CLI arguments helper."""

from plaso.lib import errors
from plaso.cli.helpers import interface


class ServerArgumentsHelper(interface.ArgumentsHelper):
  """Server configuration CLI arguments helper."""

  NAME = u'server_config'
  DESCRIPTION = u'Argument helper for a server configuration.'

  _DEFAULT_SERVER = u'127.0.0.1'
  _DEFAULT_PORT = 80

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--server', dest=u'server', type=str, action=u'store',
        default=cls._DEFAULT_SERVER, metavar=u'HOSTNAME',
        help=u'The hostname or server IP address of the server.')
    argument_group.add_argument(
        u'--port', dest=u'port', type=int, action=u'store',
        default=cls._DEFAULT_PORT, metavar=u'PORT',
        help=u'The port number of the server.')

  @classmethod
  def ParseOptions(cls, options, output_module):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      output_module (OutputModule): output module to configure.

    Raises:
      BadConfigObject: when the output module object does not have the
          SetServerInformation method.
    """
    if not hasattr(output_module, u'SetServerInformation'):
      raise errors.BadConfigObject(u'Unable to set server information.')

    server = cls._ParseStringOption(
        options, u'server', default_value=cls._DEFAULT_SERVER)
    port = cls._ParseNumericOption(
        options, u'port', default_value=cls._DEFAULT_PORT)

    output_module.SetServerInformation(server, port)



# -*- coding: utf-8 -*-
"""The Windows Restore Point (rp.log) file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class RestorePointInfoFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows Windows Restore Point information event."""

  DATA_TYPE = u'windows:restore_point:info'

  FORMAT_STRING_PIECES = [
      u'{description}',
      u'Event type: {restore_point_event_type}',
      u'Restore point type: {restore_point_type}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{description}']

  SOURCE_LONG = u'Windows Restore Point'
  SOURCE_SHORT = u'RP'

  _RESTORE_POINT_EVENT_TYPES = {
      100: u'BEGIN_SYSTEM_CHANGE',
      101: u'END_SYSTEM_CHANGE',
      102: u'BEGIN_NESTED_SYSTEM_CHANGE',
      103: u'END_NESTED_SYSTEM_CHANGE',
  }

  _RESTORE_POINT_TYPES = {
      0: u'APPLICATION_INSTALL',
      1: u'APPLICATION_UNINSTALL',
      10: u'DEVICE_DRIVER_INSTALL',
      12: u'MODIFY_SETTINGS',
      13: u'CANCELLED_OPERATION',
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    restore_point_event_type = event_values.get(
        u'restore_point_event_type', None)
    if restore_point_event_type is not None:
      event_values[u'restore_point_event_type'] = (
          self._RESTORE_POINT_EVENT_TYPES.get(
              restore_point_event_type, u'UNKNOWN'))

    restore_point_type = event_values.get(u'restore_point_type', None)
    if restore_point_type is not None:
      event_values[u'restore_point_type'] = (
          self._RESTORE_POINT_EVENT_TYPES.get(restore_point_type, u'UNKNOWN'))

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(RestorePointInfoFormatter)



# -*- coding: utf-8 -*-
"""The log2timeline CLI tool."""

from __future__ import unicode_literals

import argparse
import logging
import sys
import time
import textwrap

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.resolver import context as dfvfs_context

import plaso

# The following import makes sure the filters are registered.
from plaso import filters  # pylint: disable=unused-import

# The following import makes sure the output modules are registered.
from plaso import output  # pylint: disable=unused-import

from plaso.cli import extraction_tool
from plaso.cli import logging_filter
from plaso.cli import status_view
from plaso.cli import tool_options
from plaso.cli import tools
from plaso.cli import views
from plaso.cli.helpers import manager as helpers_manager
from plaso.engine import configurations
from plaso.engine import engine
from plaso.engine import single_process as single_process_engine
from plaso.frontend import utils as frontend_utils
from plaso.analyzers.hashers import manager as hashers_manager
from plaso.lib import definitions
from plaso.lib import errors
from plaso.multi_processing import task_engine as multi_process_engine
from plaso.parsers import manager as parsers_manager
from plaso.storage import sqlite_file as storage_sqlite_file
from plaso.storage import zip_file as storage_zip_file


class Log2TimelineTool(
    extraction_tool.ExtractionTool,
    tool_options.HashersOptions,
    tool_options.ParsersOptions,
    tool_options.StorageFileOptions):
  """Log2timeline CLI tool.

  Attributes:
    dependencies_check (bool): True if the availability and versions of
        dependencies should be checked.
    list_hashers (bool): True if the hashers should be listed.
    list_output_modules (bool): True if information about the output modules
        should be shown.
    list_parsers_and_plugins (bool): True if the parsers and plugins should
        be listed.
    show_info (bool): True if information about hashers, parsers, plugins,
        etc. should be shown.
  """

  NAME = 'log2timeline'
  DESCRIPTION = textwrap.dedent('\n'.join([
      '',
      ('log2timeline is a command line tool to extract events from '
       'individual '),
      'files, recursing a directory (e.g. mount point) or storage media ',
      'image or device.',
      '',
      'More information can be gathered from here:',
      '    https://github.com/log2timeline/plaso/wiki/Using-log2timeline',
      '']))

  EPILOG = textwrap.dedent('\n'.join([
      '',
      'Example usage:',
      '',
      'Run the tool against a storage media image (full kitchen sink)',
      '    log2timeline.py /cases/mycase/storage.plaso ímynd.dd',
      '',
      'Instead of answering questions, indicate some of the options on the',
      'command line (including data from particular VSS stores).',
      ('    log2timeline.py -o 63 --vss_stores 1,2 /cases/plaso_vss.plaso '
       'image.E01'),
      '',
      'And that is how you build a timeline using log2timeline...',
      '']))

  # The window status-view mode has an annoying flicker on Windows,
  # hence we default to linear status-view mode instead.
  if sys.platform.startswith('win'):
    _DEFAULT_STATUS_VIEW_MODE = status_view.StatusView.MODE_LINEAR
  else:
    _DEFAULT_STATUS_VIEW_MODE = status_view.StatusView.MODE_WINDOW

  _SOURCE_TYPES_TO_PREPROCESS = frozenset([
      dfvfs_definitions.SOURCE_TYPE_DIRECTORY,
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_DEVICE,
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_IMAGE])

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(Log2TimelineTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._command_line_arguments = None
    self._enable_sigsegv_handler = False
    self._number_of_extraction_workers = 0
    self._resolver_context = dfvfs_context.Context()
    self._storage_serializer_format = definitions.SERIALIZER_FORMAT_JSON
    self._source_type = None
    self._status_view = status_view.StatusView(self._output_writer, self.NAME)
    self._status_view_mode = self._DEFAULT_STATUS_VIEW_MODE
    self._stdout_output_writer = isinstance(
        self._output_writer, tools.StdoutOutputWriter)
    self._storage_file_path = None
    self._storage_format = definitions.STORAGE_FORMAT_ZIP
    self._temporary_directory = None
    self._text_prepend = None
    self._use_zeromq = True
    self._worker_memory_limit = None
    self._yara_rules_string = None

    self.dependencies_check = True
    self.list_hashers = False
    self.list_output_modules = False
    self.list_parsers_and_plugins = False
    self.show_info = False

  def _CreateProcessingConfiguration(self):
    """Creates a processing configuration.

    Returns:
      ProcessingConfiguration: processing configuration.
    """
    # TODO: pass preferred_encoding.
    configuration = configurations.ProcessingConfiguration()
    configuration.credentials = self._credential_configurations
    configuration.debug_output = self._debug_mode
    configuration.event_extraction.text_prepend = self._text_prepend
    configuration.extraction.hasher_file_size_limit = (
        self._hasher_file_size_limit)
    configuration.extraction.hasher_names_string = self._hasher_names_string
    configuration.extraction.process_archives = self._process_archives
    configuration.extraction.process_compressed_streams = (
        self._process_compressed_streams)
    configuration.extraction.yara_rules_string = self._yara_rules_string
    configuration.filter_file = self._filter_file
    configuration.input_source.mount_path = self._mount_path
    configuration.parser_filter_expression = self._parser_filter_expression
    configuration.preferred_year = self._preferred_year
    configuration.profiling.directory = self._profiling_directory
    configuration.profiling.sample_rate = self._profiling_sample_rate
    configuration.profiling.profilers = self._profilers
    configuration.temporary_directory = self._temporary_directory

    return configuration

  def _GetPluginData(self):
    """Retrieves the version and various plugin information.

    Returns:
      dict[str, list[str]]: available parsers and plugins.
    """
    return_dict = {}

    return_dict['Versions'] = [
        ('plaso engine', plaso.__version__),
        ('python', sys.version)]

    hashers_information = hashers_manager.HashersManager.GetHashersInformation()
    parsers_information = parsers_manager.ParsersManager.GetParsersInformation()
    plugins_information = (
        parsers_manager.ParsersManager.GetParserPluginsInformation())
    presets_information = self._GetParserPresetsInformation()

    return_dict['Hashers'] = hashers_information
    return_dict['Parsers'] = parsers_information
    return_dict['Parser Plugins'] = plugins_information
    return_dict['Parser Presets'] = presets_information

    return return_dict

  def _ParseProcessingOptions(self, options):
    """Parses the processing options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._use_zeromq = getattr(options, 'use_zeromq', True)

    self._single_process_mode = getattr(options, 'single_process', False)

    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=['temporary_directory'])

    self._worker_memory_limit = getattr(options, 'worker_memory_limit', None)
    self._number_of_extraction_workers = getattr(options, 'workers', 0)

    # TODO: add code to parse the worker options.

  def AddProcessingOptions(self, argument_group):
    """Adds the processing options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        '--disable_zeromq', '--disable-zeromq', action='store_false',
        dest='use_zeromq', default=True, help=(
            'Disable queueing using ZeroMQ. A Multiprocessing queue will be '
            'used instead.'))

    argument_group.add_argument(
        '--single_process', '--single-process', dest='single_process',
        action='store_true', default=False, help=(
            'Indicate that the tool should run in a single process.'))

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        argument_group, names=['temporary_directory'])

    argument_group.add_argument(
        '--worker-memory-limit', '--worker_memory_limit',
        dest='worker_memory_limit', action='store', type=int,
        metavar='SIZE', help=(
            'Maximum amount of memory a worker process is allowed to consume. '
            '[defaults to 2 GiB]'))

    argument_group.add_argument(
        '--workers', dest='workers', action='store', type=int, default=0,
        help=('The number of worker processes [defaults to available system '
              'CPUs minus one].'))

  def ParseArguments(self):
    """Parses the command line arguments.

    Returns:
      bool: True if the arguments were successfully parsed.
    """
    self._ConfigureLogging()

    argument_parser = argparse.ArgumentParser(
        description=self.DESCRIPTION, epilog=self.EPILOG, add_help=False,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    self.AddBasicOptions(argument_parser)

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        argument_parser, names=['storage_file', 'storage_format'])

    extraction_group = argument_parser.add_argument_group(
        'Extraction Arguments')

    argument_helper_names = [
        'artifact_definitions', 'extraction', 'filter_file', 'hashers',
        'parsers', 'yara_rules']
    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        extraction_group, names=argument_helper_names)

    self.AddStorageMediaImageOptions(extraction_group)
    self.AddTimeZoneOption(extraction_group)
    self.AddVSSProcessingOptions(extraction_group)
    self.AddCredentialOptions(extraction_group)

    info_group = argument_parser.add_argument_group('Informational Arguments')

    self.AddInformationalOptions(info_group)

    info_group.add_argument(
        '--info', dest='show_info', action='store_true', default=False,
        help='Print out information about supported plugins and parsers.')

    info_group.add_argument(
        '--use_markdown', '--use-markdown', dest='use_markdown',
        action='store_true', default=False, help=(
            'Output lists in Markdown format use in combination with '
            '"--hashers list", "--parsers list" or "--timezone list"'))

    info_group.add_argument(
        '--no_dependencies_check', '--no-dependencies-check',
        dest='dependencies_check', action='store_false', default=True,
        help='Disable the dependencies check.')

    self.AddLogFileOptions(info_group)

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        info_group, names=['status_view'])

    output_group = argument_parser.add_argument_group('Output Arguments')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        output_group, names=['text_prepend'])

    processing_group = argument_parser.add_argument_group(
        'Processing Arguments')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        processing_group, names=['data_location'])

    self.AddPerformanceOptions(processing_group)
    self.AddProfilingOptions(processing_group)
    self.AddProcessingOptions(processing_group)

    processing_group.add_argument(
        '--sigsegv_handler', '--sigsegv-handler', dest='sigsegv_handler',
        action='store_true', default=False, help=(
            'Enables the SIGSEGV handler. WARNING this functionality is '
            'experimental and will a deadlock worker process if a real '
            'segfault is caught, but not signal SIGSEGV. This functionality '
            'is therefore primarily intended for debugging purposes'))

    argument_parser.add_argument(
        self._SOURCE_OPTION, action='store', metavar='SOURCE', nargs='?',
        default=None, type=str, help=(
            'The path to the source device, file or directory. If the source '
            'is a supported storage media device or image file, archive file '
            'or a directory, the files within are processed recursively.'))

    try:
      options = argument_parser.parse_args()
    except UnicodeEncodeError:
      # If we get here we are attempting to print help in a non-Unicode
      # terminal.
      self._output_writer.Write('\n')
      self._output_writer.Write(argument_parser.format_help())
      return False

    # Properly prepare the attributes according to local encoding.
    if self.preferred_encoding == 'ascii':
      logging.warning(
          'The preferred encoding of your system is ASCII, which is not '
          'optimal for the typically non-ASCII characters that need to be '
          'parsed and processed. The tool will most likely crash and die, '
          'perhaps in a way that may not be recoverable. A five second delay '
          'is introduced to give you time to cancel the runtime and '
          'reconfigure your preferred encoding, otherwise continue at own '
          'risk.')
      time.sleep(5)

    if self._process_archives:
      logging.warning(
          'Scanning archive files currently can cause deadlock. Continue at '
          'your own risk.')
      time.sleep(5)

    try:
      self.ParseOptions(options)
    except errors.BadConfigOption as exception:
      self._output_writer.Write('ERROR: {0!s}\n'.format(exception))
      self._output_writer.Write('\n')
      self._output_writer.Write(argument_parser.format_usage())
      return False

    self._command_line_arguments = self.GetCommandLineArguments()

    return True

  def ParseOptions(self, options):
    """Parses the options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    # The extraction options are dependent on the data location.
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=['data_location'])

    # Check the list options first otherwise required options will raise.
    argument_helper_names = ['hashers', 'parsers']
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=argument_helper_names)

    self._ParseProfilingOptions(options)
    self._ParseTimezoneOption(options)

    self.list_hashers = self._hasher_names_string == 'list'
    self.list_parsers_and_plugins = self._parser_filter_expression == 'list'

    self.show_info = getattr(options, 'show_info', False)

    if getattr(options, 'use_markdown', False):
      self._views_format_type = views.ViewsFactory.FORMAT_TYPE_MARKDOWN

    self.dependencies_check = getattr(options, 'dependencies_check', True)

    if (self.list_hashers or self.list_parsers_and_plugins or
        self.list_profilers or self.list_timezones or self.show_info):
      return

    self._ParseInformationalOptions(options)

    argument_helper_names = [
        'artifact_definitions', 'extraction', 'filter_file', 'status_view',
        'storage_file', 'storage_format', 'text_prepend']
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=argument_helper_names)

    self._ParseLogFileOptions(options)

    self._ParseStorageMediaOptions(options)

    self._ParsePerformanceOptions(options)
    self._ParseProcessingOptions(options)

    format_string = (
        '%(asctime)s [%(levelname)s] (%(processName)-10s) PID:%(process)d '
        '<%(module)s> %(message)s')

    if self._debug_mode:
      logging_level = logging.DEBUG
    elif self._quiet_mode:
      logging_level = logging.WARNING
    else:
      logging_level = logging.INFO

    self._ConfigureLogging(
        filename=self._log_file, format_string=format_string,
        log_level=logging_level)

    if self._debug_mode:
      log_filter = logging_filter.LoggingFilter()
      root_logger = logging.getLogger()
      root_logger.addFilter(log_filter)

    if not self._storage_file_path:
      raise errors.BadConfigOption('Missing storage file option.')

    serializer_format = getattr(
        options, 'serializer_format', definitions.SERIALIZER_FORMAT_JSON)
    if serializer_format not in definitions.SERIALIZER_FORMATS:
      raise errors.BadConfigOption(
          'Unsupported storage serializer format: {0:s}.'.format(
              serializer_format))
    self._storage_serializer_format = serializer_format

    # TODO: where is this defined?
    self._operating_system = getattr(options, 'os', None)

    if self._operating_system:
      self._mount_path = getattr(options, 'filename', None)

    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=['status_view'])

    self._enable_sigsegv_handler = getattr(options, 'sigsegv_handler', False)

  def _PreprocessSources(self, extraction_engine):
    """Preprocesses the sources.

    Args:
      extraction_engine (BaseEngine): extraction engine to preprocess
          the sources.
    """
    logging.debug('Starting preprocessing.')

    try:
      extraction_engine.PreprocessSources(
          self._artifacts_registry, self._source_path_specs,
          resolver_context=self._resolver_context)

    except IOError as exception:
      logging.error('Unable to preprocess with error: {0:s}'.format(exception))

    logging.debug('Preprocessing done.')

  def ExtractEventsFromSources(self):
    """Processes the sources and extracts events.

    Raises:
      BadConfigOption: if the storage file path is invalid.
      SourceScannerError: if the source scanner could not find a supported
          file system.
      UserAbort: if the user initiated an abort.
    """
    self._CheckStorageFile(self._storage_file_path)

    scan_context = self.ScanSource()
    self._source_type = scan_context.source_type

    self._status_view.SetMode(self._status_view_mode)
    self._status_view.SetSourceInformation(
        self._source_path, self._source_type, filter_file=self._filter_file)

    status_update_callback = (
        self._status_view.GetExtractionStatusUpdateCallback())

    self._output_writer.Write('\n')
    self._status_view.PrintExtractionStatusHeader(None)
    self._output_writer.Write('Processing started.\n')

    session = engine.BaseEngine.CreateSession(
        command_line_arguments=self._command_line_arguments,
        debug_mode=self._debug_mode,
        filter_file=self._filter_file,
        preferred_encoding=self.preferred_encoding,
        preferred_time_zone=self._preferred_time_zone,
        preferred_year=self._preferred_year)

    if self._storage_format == definitions.STORAGE_FORMAT_SQLITE:
      storage_writer = storage_sqlite_file.SQLiteStorageFileWriter(
          session, self._storage_file_path)

    else:
      storage_writer = storage_zip_file.ZIPStorageFileWriter(
          session, self._storage_file_path)

    single_process_mode = self._single_process_mode
    if self._source_type == dfvfs_definitions.SOURCE_TYPE_FILE:
      # No need to multi process a single file source.
      single_process_mode = True

    if single_process_mode:
      extraction_engine = single_process_engine.SingleProcessEngine()
    else:
      extraction_engine = multi_process_engine.TaskMultiProcessEngine(
          use_zeromq=self._use_zeromq)

    # If the source is a directory or a storage media image
    # run pre-processing.
    if (self._force_preprocessing or
        self._source_type in self._SOURCE_TYPES_TO_PREPROCESS):
      self._PreprocessSources(extraction_engine)

    configuration = self._CreateProcessingConfiguration()

    if not configuration.parser_filter_expression:
      operating_system = extraction_engine.knowledge_base.GetValue(
          'operating_system')
      operating_system_product = extraction_engine.knowledge_base.GetValue(
          'operating_system_product')
      operating_system_version = extraction_engine.knowledge_base.GetValue(
          'operating_system_version')
      parser_filter_expression = (
          parsers_manager.ParsersManager.GetPresetForOperatingSystem(
              operating_system, operating_system_product,
              operating_system_version))

      if parser_filter_expression:
        logging.info('Parser filter expression changed to: {0:s}'.format(
            parser_filter_expression))

      configuration.parser_filter_expression = parser_filter_expression

      names_generator = parsers_manager.ParsersManager.GetParserAndPluginNames(
          parser_filter_expression=parser_filter_expression)

      session.enabled_parser_names = list(names_generator)
      session.parser_filter_expression = parser_filter_expression

    # Note session.preferred_time_zone will default to UTC but
    # self._preferred_time_zone is None when not set.
    if self._preferred_time_zone:
      try:
        extraction_engine.knowledge_base.SetTimeZone(self._preferred_time_zone)
      except ValueError:
        # pylint: disable=protected-access
        logging.warning(
            'Unsupported time zone: {0:s}, defaulting to {1:s}'.format(
                self._preferred_time_zone,
                extraction_engine.knowledge_base._time_zone.zone))

    filter_find_specs = None
    if configuration.filter_file:
      environment_variables = (
          extraction_engine.knowledge_base.GetEnvironmentVariables())
      filter_find_specs = frontend_utils.BuildFindSpecsFromFile(
          configuration.filter_file,
          environment_variables=environment_variables)

    processing_status = None
    if single_process_mode:
      logging.debug('Starting extraction in single process mode.')

      processing_status = extraction_engine.ProcessSources(
          self._source_path_specs, storage_writer, self._resolver_context,
          configuration, filter_find_specs=filter_find_specs,
          status_update_callback=status_update_callback)

    else:
      logging.debug('Starting extraction in multi process mode.')

      processing_status = extraction_engine.ProcessSources(
          session.identifier, self._source_path_specs, storage_writer,
          configuration, enable_sigsegv_handler=self._enable_sigsegv_handler,
          filter_find_specs=filter_find_specs,
          number_of_worker_processes=self._number_of_extraction_workers,
          status_update_callback=status_update_callback,
          worker_memory_limit=self._worker_memory_limit)

    self._status_view.PrintExtractionSummary(processing_status)

  def ShowInfo(self):
    """Shows information about available hashers, parsers, plugins, etc."""
    self._output_writer.Write(
        '{0:=^80s}\n'.format(' log2timeline/plaso information '))

    plugin_list = self._GetPluginData()
    for header, data in plugin_list.items():
      table_view = views.ViewsFactory.GetTableView(
          self._views_format_type, column_names=['Name', 'Description'],
          title=header)
      for entry_header, entry_data in sorted(data):
        table_view.AddRow([entry_header, entry_data])
      table_view.Write(self._output_writer)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the MSIE WebCache ESE database event formatter."""

import unittest

from plaso.formatters import msie_webcache

from tests.formatters import test_lib


class MsieWebCacheContainerEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the WebCache Container_# table record event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = msie_webcache.MsieWebCacheContainerEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = msie_webcache.MsieWebCacheContainerEventFormatter()

    expected_attribute_names = [
        u'entry_identifier',
        u'container_identifier',
        u'cache_identifier',
        u'url',
        u'redirect_url',
        u'access_count',
        u'sync_count',
        u'cached_filename',
        u'file_extension',
        u'cached_file_size',
        u'request_headers',
        u'response_headers']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class MsieWebCacheContainersEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the WebCache Containers table record event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = msie_webcache.MsieWebCacheContainersEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = msie_webcache.MsieWebCacheContainersEventFormatter()

    expected_attribute_names = [
        u'container_identifier',
        u'set_identifier',
        u'name',
        u'directory']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The selinux event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class SELinuxFormatter(interface.ConditionalEventFormatter):
  """Formatter for a selinux log file event."""

  DATA_TYPE = u'selinux:line'

  FORMAT_STRING_SEPARATOR = u''

  FORMAT_STRING_PIECES = [
      u'[',
      u'audit_type: {audit_type}',
      u', pid: {pid}',
      u']',
      u' {body}']

  SOURCE_LONG = u'Audit log File'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(SELinuxFormatter)



# -*- coding: utf-8 -*-
"""The SQLite parser plugin interface."""

import logging
import sys

# pylint: disable=wrong-import-order
try:
  from pysqlite2 import dbapi2 as sqlite3
except ImportError:
  import sqlite3

from plaso.parsers import plugins


class SQLitePlugin(plugins.BasePlugin):
  """SQLite parser plugin."""

  NAME = u'sqlite'
  DESCRIPTION = u'Parser for SQLite database files.'

  # Queries to be executed.
  # Should be a list of tuples with two entries, SQLCommand and callback
  # function name.
  QUERIES = []

  # List of tables that should be present in the database, for verification.
  REQUIRED_TABLES = frozenset([])

  # Database schemas this plugin was originally designed for.
  # Should be a list of dictionaries with {table_name: SQLCommand} format.
  SCHEMAS = []

  @classmethod
  def _HashRow(cls, row):
    """Hashes the given row.

    Args:
      row (sqlite3.Row): row.

    Returns:
      int: hash value of the given row.
    """
    hash_value = 0
    for column_value in row:
      # In Python 2, blobs are "read-write buffer" and will cause a
      # "writable buffers are not hashable" error if we try to hash it.
      # Therefore, we will turn it into a string beforehand.
      if sys.version_info[0] < 3 and isinstance(column_value, buffer):
        column_value = str(column_value)
      hash_value ^= hash(column_value)
    return hash_value

  def _ParseQuery(
      self, parser_mediator, database, query, callback, row_cache, cache=None):
    """Queries a database and parses the results.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      database (SQLiteDatabase): database.
      query (str): query.
      callback (function): function to invoke to parse an individual row.
      row_cache (set): hashes of the rows that have been parsed.
      cache (Optional[SQLiteCache]): cache.
    """
    try:
      rows = database.Query(query)

    except sqlite3.DatabaseError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to run query: {0:s} on database with error: {1!s}'.format(
              query, exception))
      return

    for index, row in enumerate(rows):
      if parser_mediator.abort:
        break

      try:
        callback(
            parser_mediator, row, cache=cache, database=database, query=query)

        row_hash = self._HashRow(row)
        row_cache.add(row_hash)

      except sqlite3.DatabaseError as exception:
        parser_mediator.ProduceExtractionError((
            u'unable to parse row: {0:d} of results of query: {1:s} on '
            u'database with error: {2!s}').format(index, query, exception))
        # TODO: consider removing return.
        return

  def _ParseQueryWithWAL(
      self, parser_mediator, database_wal, query, callback, row_cache,
      cache=None):
    """Queries a database with WAL and parses the results.

    Note that cached rows will be ignored.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      database_wal (SQLiteDatabase): database with WAL.
      query (str): query.
      callback (function): function to invoke to parse an individual row.
      row_cache (set): hashes of the rows that have been parsed.
      cache (Optional[SQLiteCache]): cache.
    """
    try:
      rows = database_wal.Query(query)

    except sqlite3.DatabaseError as exception:
      parser_mediator.ProduceExtractionError((
          u'unable to run query: {0:s} on database with WAL with error: '
          u'{1!s}').format(query, exception))
      return

    for index, row in enumerate(rows):
      if parser_mediator.abort:
        break

      row_hash = self._HashRow(row)
      if row_hash in row_cache:
        continue

      try:
        callback(
            parser_mediator, row, cache=cache, database=database_wal,
            query=query)

      except sqlite3.DatabaseError as exception:
        parser_mediator.ProduceExtractionError((
            u'unable to parse row: {0:d} of results of query: {1:s} on '
            u'database with WAL with error: {2!s}').format(
                index, query, exception))
        # TODO: consider removing return.
        return

  def GetEntries(
      self, parser_mediator, cache=None, database=None, database_wal=None,
      wal_file_entry=None, **unused_kwargs):
    """Extracts events from a SQLite database.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      cache (Optional[SQLiteCache]): cache.
      database (Optional[SQLiteDatabase]): database.
      database_wal (Optional[SQLiteDatabase]): database object with WAL file
          commited.
      wal_file_entry (Optional[dfvfs.FileEntry]): file entry for the database
          with WAL file commited.
    """
    schema_match = None
    if database:
      schema_match = any(schema == database.schema for schema in self.SCHEMAS)

    wal_schema_match = None
    if database_wal:
      wal_schema_match = any(
          schema == database_wal.schema for schema in self.SCHEMAS)

    for query, callback_method in self.QUERIES:
      if parser_mediator.abort:
        break

      callback = getattr(self, callback_method, None)
      if callback is None:
        logging.warning(
            u'[{0:s}] missing callback method: {1:s} for query: {2:s}'.format(
                self.NAME, callback_method, query))
        continue

      row_cache = set()

      if database:
        try:
          parser_mediator.AddEventAttribute(u'schema_match', schema_match)

          self._ParseQuery(
              parser_mediator, database, query, callback, row_cache,
              cache=cache)

        finally:
          parser_mediator.RemoveEventAttribute(u'schema_match')

      if database_wal:
        file_entry = parser_mediator.GetFileEntry()

        try:
          parser_mediator.AddEventAttribute(u'schema_match', wal_schema_match)
          parser_mediator.SetFileEntry(wal_file_entry)

          self._ParseQueryWithWAL(
              parser_mediator, database_wal, query, callback, row_cache,
              cache=cache)

        finally:
          parser_mediator.RemoveEventAttribute(u'schema_match')
          parser_mediator.SetFileEntry(file_entry)

  def Process(
      self, parser_mediator, cache=None, database=None, database_wal=None,
      wal_file_entry=None, **kwargs):
    """Determine if this is the right plugin for this database.

    This function takes a SQLiteDatabase object and compares the list
    of required tables against the available tables in the database.
    If all the tables defined in REQUIRED_TABLES are present in the
    database then this plugin is considered to be the correct plugin
    and the function will return back a generator that yields event
    objects.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.
      database_wal (Optional[SQLiteDatabase]): database with its
          Write-Ahead Log (WAL) committed.
      wal_file_entry (Optional[dfvfs.FileEntry]): file entry of the database
          with is Write-Ahead Log (WAL) committed.

    Raises:
      ValueError: If the database attribute is not passed in.
    """
    if database is None:
      raise ValueError(u'Database is not set.')

    # This will raise if unhandled keyword arguments are passed.
    super(SQLitePlugin, self).Process(parser_mediator)

    self.GetEntries(
        parser_mediator, cache=cache, database=database,
        database_wal=database_wal, wal_file_entry=wal_file_entry)



# -*- coding: utf-8 -*-
"""The shutdown Windows Registry event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class ShutdownWindowsRegistryEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a shutdown Windows Registry event."""

  DATA_TYPE = u'windows:registry:shutdown'

  FORMAT_STRING_PIECES = [
      u'[{key_path}]',
      u'Description: {value_name}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{value_name}']

  SOURCE_LONG = u'Registry Key Shutdown Entry'
  SOURCE_SHORT = u'REG'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    regvalue = event_values.get(u'regvalue', {})
    string_parts = []
    for key, value in sorted(regvalue.items()):
      string_parts.append(u'{0:s}: {1!s}'.format(key, value))
    event_values[u'text'] = u' '.join(string_parts)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(
    ShutdownWindowsRegistryEventFormatter)



# -*- coding: utf-8 -*-
"""Parser for Extensible Storage Engine (ESE) database files (EDB)."""

import pyesedb

from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager
from plaso.parsers import plugins


class ESEDBCache(plugins.BasePluginCache):
  """A cache storing query results for ESEDB plugins."""

  def StoreDictInCache(self, attribute_name, dict_object):
    """Store a dict object in cache.

    Args:
      attribute_name (str): name of the attribute.
      dict_object (dict): dictionary.
    """
    setattr(self, attribute_name, dict_object)


class ESEDBParser(interface.FileObjectParser):
  """Parses Extensible Storage Engine (ESE) database files (EDB)."""

  _INITIAL_FILE_OFFSET = None

  NAME = u'esedb'
  DESCRIPTION = u'Parser for Extensible Storage Engine (ESE) database files.'

  _plugin_classes = {}

  def _GetTableNames(self, database):
    """Retrieves the table names in a database.

    Args:
      database (pyesedb.file): ESE database.

    Returns:
      list[str]: table names.
    """
    table_names = []
    for esedb_table in database.tables:
      table_names.append(esedb_table.name)

    return table_names

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'\xef\xcd\xab\x89', offset=4)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses an ESE database file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
    """
    esedb_file = pyesedb.file()

    try:
      esedb_file.open_file_object(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to open file with error: {0:s}'.format(exception))
      return

    # Compare the list of available plugin objects.
    cache = ESEDBCache()
    try:
      table_names = frozenset(self._GetTableNames(esedb_file))

      for plugin in self._plugins:
        if parser_mediator.abort:
          break

        if plugin.required_tables.difference(table_names):
          continue

        try:
          plugin.UpdateChainAndProcess(
              parser_mediator, cache=cache, database=esedb_file)

        except Exception as exception:  # pylint: disable=broad-except
          parser_mediator.ProduceExtractionError((
              u'plugin: {0:s} unable to parse ESE database with error: '
              u'{1:s}').format(plugin.NAME, exception))

    finally:
      # TODO: explicitly clean up cache.

      esedb_file.close()


manager.ParsersManager.RegisterParser(ESEDBParser)



# -*- coding: utf-8 -*-
"""The Mac OS X launch services (LS) quarantine event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class LSQuarantineFormatter(interface.ConditionalEventFormatter):
  """Formatter for a launch services (LS) quarantine history event."""

  DATA_TYPE = u'macosx:lsquarantine'

  FORMAT_STRING_PIECES = [
      u'[{agent}]',
      u'Downloaded: {url}',
      u'<{data}>']

  FORMAT_STRING_SHORT_PIECES = [u'{url}']

  SOURCE_LONG = u'LS Quarantine Event'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(LSQuarantineFormatter)



# -*- coding: utf-8 -*-
"""The temporary directory CLI arguments helper."""

import os

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class TemporaryDirectoryArgumentsHelper(interface.ArgumentsHelper):
  """Temporary directory CLI arguments helper."""

  NAME = u'temporary_directory'
  DESCRIPTION = u'Temporary directory command line arguments.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--temporary_directory', u'--temporary-directory',
        dest=u'temporary_directory', type=str, action=u'store',
        metavar=u'DIRECTORY', help=(
            u'Path to the directory that should be used to store temporary '
            u'files created during processing.'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          u'Configuration object is not an instance of CLITool')

    temporary_directory = getattr(options, u'temporary_directory', None)
    if temporary_directory and not os.path.isdir(temporary_directory):
      raise errors.BadConfigOption(
          u'No such temporary directory: {0:s}'.format(temporary_directory))

    setattr(configuration_object, u'_temporary_directory', temporary_directory)


manager.ArgumentHelperManager.RegisterHelper(TemporaryDirectoryArgumentsHelper)



# -*- coding: utf-8 -*-
"""The text file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class TextEntryFormatter(interface.EventFormatter):
  """Formatter for a text file entry event."""

  DATA_TYPE = u'text:entry'
  FORMAT_STRING = u'{text}'

  SOURCE_SHORT = u'LOG'
  SOURCE_LONG = u'Text File'


manager.FormattersManager.RegisterFormatter(TextEntryFormatter)



# -*- coding: utf-8 -*-
"""This file contains a MRUList Registry plugin."""

import abc
import logging

import construct

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import definitions
from plaso.lib import binary
from plaso.parsers import winreg
from plaso.parsers.shared import shell_items
from plaso.parsers.winreg_plugins import interface


class MRUListStringRegistryKeyFilter(
    interface.WindowsRegistryKeyWithValuesFilter):
  """Windows Registry key with values filter."""

  _IGNORE_KEY_PATH_SUFFIXES = frozenset([
      u'\\Explorer\\DesktopStreamMRU'.upper()])

  _VALUE_NAMES = (u'a', u'MRUList')

  def __init__(self):
    """Initializes a Windows Registry key filter object."""
    super(MRUListStringRegistryKeyFilter, self).__init__(self._VALUE_NAMES)

  def Match(self, registry_key):
    """Determines if a Windows Registry key matches the filter.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the Windows Registry key matches the filter.
    """
    key_path = registry_key.path.upper()
    # Prevent this filter matching non-string MRUList values.
    for ignore_key_path_suffix in self._IGNORE_KEY_PATH_SUFFIXES:
      if key_path.endswith(ignore_key_path_suffix):
        return False

    return super(MRUListStringRegistryKeyFilter, self).Match(registry_key)


class BaseMRUListPlugin(interface.WindowsRegistryPlugin):
  """Class for common MRUList Windows Registry plugin functionality."""

  _MRULIST_STRUCT = construct.Range(1, 500, construct.ULInt16(u'entry_letter'))

  _SOURCE_APPEND = u': MRU List'

  @abc.abstractmethod
  def _ParseMRUListEntryValue(
      self, parser_mediator, registry_key, entry_index, entry_letter, **kwargs):
    """Parses the MRUList entry value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUList value.
      entry_index (int): MRUList entry index.
      entry_letter (str): character value representing the entry.

    Returns:
      str: MRUList entry value.
    """

  def _ParseMRUListValue(self, registry_key):
    """Parses the MRUList value in a given Registry key.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUList value.

    Returns:
      generator: MRUList value generator, which returns the MRU index number
          and entry value.
    """
    mru_list_value = registry_key.GetValueByName(u'MRUList')

    # The key exists but does not contain a value named "MRUList".
    if not mru_list_value:
      return enumerate([])

    try:
      mru_list = self._MRULIST_STRUCT.parse(mru_list_value.data)
    except construct.FieldError:
      logging.warning(u'[{0:s}] Unable to parse the MRU key: {1:s}'.format(
          self.NAME, registry_key.path))
      return enumerate([])

    return enumerate(mru_list)

  def _ParseMRUListKey(self, parser_mediator, registry_key, codepage=u'cp1252'):
    """Extract event objects from a MRUList Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      codepage (Optional[str]): extended ASCII string codepage.
    """
    values_dict = {}
    for entry_index, entry_letter in self._ParseMRUListValue(registry_key):
      # TODO: detect if list ends prematurely.
      # MRU lists are terminated with \0 (0x0000).
      if entry_letter == 0:
        break

      entry_letter = chr(entry_letter)

      value_string = self._ParseMRUListEntryValue(
          parser_mediator, registry_key, entry_index, entry_letter,
          codepage=codepage)

      value_text = u'Index: {0:d} [MRU Value {1:s}]'.format(
          entry_index + 1, entry_letter)

      values_dict[value_text] = value_string

    event_data = windows_events.WindowsRegistryEventData()
    event_data.key_path = registry_key.path
    event_data.offset = registry_key.offset
    event_data.regvalue = values_dict
    event_data.source_append = self._SOURCE_APPEND

    event = time_events.DateTimeValuesEvent(
        registry_key.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)


class MRUListStringPlugin(BaseMRUListPlugin):
  """Windows Registry plugin to parse a string MRUList."""

  NAME = u'mrulist_string'
  DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'

  FILTERS = frozenset([MRUListStringRegistryKeyFilter()])

  URLS = [u'http://forensicartifacts.com/tag/mru/']

  def _ParseMRUListEntryValue(
      self, parser_mediator, registry_key, entry_index, entry_letter, **kwargs):
    """Parses the MRUList entry value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUList value.
      entry_index (int): MRUList entry index.
      entry_letter (str): character value representing the entry.

    Returns:
      str: MRUList entry value.
    """
    value_string = u''

    value = registry_key.GetValueByName(u'{0:s}'.format(entry_letter))
    if value is None:
      logging.debug(
          u'[{0:s}] Missing MRUList entry value: {1:s} in key: {2:s}.'.format(
              self.NAME, entry_letter, registry_key.path))

    elif value.DataIsString():
      value_string = value.GetDataAsObject()

    elif value.DataIsBinaryData():
      logging.debug((
          u'[{0:s}] Non-string MRUList entry value: {1:s} parsed as string '
          u'in key: {2:s}.').format(self.NAME, entry_letter, registry_key.path))
      utf16_stream = binary.ByteStreamCopyToUTF16Stream(value.data)

      try:
        value_string = utf16_stream.decode(u'utf-16-le')
      except UnicodeDecodeError as exception:
        value_string = binary.HexifyBuffer(utf16_stream)
        logging.warning((
            u'[{0:s}] Unable to decode UTF-16 stream: {1:s} in MRUList entry '
            u'value: {2:s} in key: {3:s} with error: {4:s}').format(
                self.NAME, value_string, entry_letter, registry_key.path,
                exception))

    return value_string

  def ExtractEvents(
      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      codepage (Optional[str]): extended ASCII string codepage.
    """
    self._ParseMRUListKey(parser_mediator, registry_key, codepage=codepage)


class MRUListShellItemListPlugin(BaseMRUListPlugin):
  """Windows Registry plugin to parse a shell item list MRUList."""

  NAME = u'mrulist_shell_item_list'
  DESCRIPTION = u'Parser for Most Recently Used (MRU) Registry data.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
          u'Explorer\\DesktopStreamMRU')])

  URLS = [u'https://github.com/libyal/winreg-kb/wiki/MRU-keys']

  def _ParseMRUListEntryValue(
      self, parser_mediator, registry_key, entry_index, entry_letter,
      codepage=u'cp1252', **kwargs):
    """Parses the MRUList entry value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUList value.
      entry_index (int): MRUList entry index.
      entry_letter (str): character value representing the entry.
      codepage (Optional[str]): extended ASCII string codepage.

    Returns:
      str: MRUList entry value.
    """
    value_string = u''

    value = registry_key.GetValueByName(u'{0:s}'.format(entry_letter))
    if value is None:
      logging.debug(
          u'[{0:s}] Missing MRUList entry value: {1:s} in key: {2:s}.'.format(
              self.NAME, entry_letter, registry_key.path))

    elif not value.DataIsBinaryData():
      logging.debug((
          u'[{0:s}] Non-binary MRUList entry value: {1:s} in key: '
          u'{2:s}.').format(self.NAME, entry_letter, registry_key.path))

    elif value.data:
      shell_items_parser = shell_items.ShellItemsParser(registry_key.path)
      shell_items_parser.ParseByteStream(
          parser_mediator, value.data, codepage=codepage)

      value_string = u'Shell item path: {0:s}'.format(
          shell_items_parser.CopyToPath())

    return value_string

  def ExtractEvents(
      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      codepage (Optional[str]): extended ASCII string codepage.
    """
    self._ParseMRUListKey(parser_mediator, registry_key, codepage=codepage)


winreg.WinRegistryParser.RegisterPlugins([
    MRUListStringPlugin, MRUListShellItemListPlugin])
